﻿using isosoft.root;
using System;
using System.Collections.ObjectModel;
using System.Windows;

namespace PolygonOptimizer
{
	public class ZoomPolygon : XObjectNotifier
	{
		ObservableCollection<Point>		_points				= new ObservableCollection<Point>();
		ZoomPortView					_portView			= null;
		ObservableCollection<Point>		_pointsOutOfPortView = new ObservableCollection<Point>();

		public ZoomPolygon()
		{
		}

		public ZoomPolygon(ZoomPortView portView)
		{
			_portView	= portView;
		}

		public ZoomPortView PortView
		{
			get {  return _portView; }
			set
			{
				if( value == _portView)
					return;

				_portView = value;
				NotifyPropertyChanged(() => PortView);
			}
		}


		public ObservableCollection<Point> Points
		{
			get {  return _points; }
			set
			{
				if( value == null ||value == _points)
					return;

				_points = value;
				NotifyPropertyChanged(() => Points);
				NotifyPropertyChanged(() => TotalPointsCount);
			}
		}


		public ObservableCollection<Point> PointsOutOfPortView
		{
			get { return _pointsOutOfPortView; }
			set
			{
				if (value == null || value == _pointsOutOfPortView)
					return;

				_pointsOutOfPortView = value;
				NotifyPropertyChanged(() => PointsOutOfPortView);
				NotifyPropertyChanged(() => TotalPointsCount);
			}
		}

		public int TotalPointsCount
		{
			get {  return (_points == null ? 0 : _points.Count) + (_pointsOutOfPortView == null ? 0 : _pointsOutOfPortView.Count); }
		}


		public static ZoomPolygon Parse(Point[] points, ZoomPortView portView)
		{
			ZoomPolygon					polygon				= new ZoomPolygon(portView);
			ObservableCollection<Point>	pointList			= new ObservableCollection<Point>();
			ObservableCollection<Point>	pointsOutPortView	= new ObservableCollection<Point>();
			double						zoomRatio			= portView == null ? 0.0 : portView.ZoomRatio;

			if (portView == null || points == null || zoomRatio < 0.0)
				return polygon;

			double	maxX	= portView.Width,
					maxY	= portView.Height;
			Point	point,
					last	= new Point();

			for(int index = 0; index< points.Length; index++)
			{
				point = points[index];

				if( ! portView.IsPointInPortview(point))
				{
					pointsOutPortView.Add(point);
					goto next_point;
				}

				if (pointList.Count <= 0)
				{
					pointList.Add(point);
					goto next_point;
				}

				if( Math.Abs( (point.X - last.X) / maxX) >= zoomRatio || Math.Abs( (point.Y - last.Y) / maxY) >= zoomRatio)
					pointList.Add(point);
next_point:
				last = point;
			}

			polygon.Points = pointList;
			polygon.PointsOutOfPortView = pointsOutPortView;
			return polygon;
		}

	}


}
