﻿using isosoft.root;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;

namespace PolygonOptimizer
{
	public partial class ZoomPolygonOptimizeTest : XObjectNotifier
	{
		protected static ZoomPolygonOptimizeTest	_instance	= null;

		/* ***********************************************
		zooms (US)
		
		Zoom 1 (lon/lat)
		***************
		-161.204, 75.477
		169.126, -51.229

		Zoom 2 (lon/lat)
		***************
		 -154.009, 67.845
		84.349, -30.517

		Zoom 3 (lon/lat)
		***************
		 -154.185, 59.671
		 0.853, -1.837

		Zoom 4 (lon/lat)
		***************
		 -128.872, 49.501
		 -65.195, 29.388
		 * 
		 * 
		 -120.149085, 47.075
		 -93.430335, 38.399
		*********************************************** */

		const	double					_defaultRatio	= 0.00021;
		private double					_zoomRatio		= _defaultRatio;
		private List<ZoomPortView>		_zoomPortViews	= new List<ZoomPortView>() 
		{
			//  (lat/lon)
			//new ZoomPortView("Zoom level 1",	new Point( 75.477, -161.204),	new Point(-51.229, 169.126),	_defaultRatio),
			//new ZoomPortView("Zoom level 2",	new Point( 67.845, -154.009),	new Point(-30.517, 84.349),		_defaultRatio),
			//new ZoomPortView("Zoom level 3 US",	new Point( 59.671, -154.185),	new Point(-1.837,   0.853),		_defaultRatio),
			//new ZoomPortView("Zoom level 4 US",	new Point( 49.501, -128.872),	new Point(29.388, -65.195),		_defaultRatio),
			//new ZoomPortView("Zoom level 5 US-full",	new Point( 47.075, -120.149085),	new Point(38.399, -93.430335),		_defaultRatio)

			//  (lon/lat)
			new ZoomPortView("Zoom level 1",	new Point( -161.204, 75.477),	new Point(169.126, -51.229),	_defaultRatio),
			new ZoomPortView("Zoom level 2",	new Point( -154.009, 67.845),	new Point(84.349, -30.517),		_defaultRatio),
			new ZoomPortView("Zoom level 3 US",	new Point( -154.185, 59.671),	new Point( 0.853, -1.837),		_defaultRatio),
			new ZoomPortView("Zoom level 4 US",	new Point( -128.872, 49.501),	new Point(-65.195, 29.388),		_defaultRatio),
			new ZoomPortView("Zoom level 5 US-full",	new Point( -120.149085, 47.075),	new Point(-93.430335, 38.399),		_defaultRatio)

		};

		ZoomPortView	_currentZoomLevel;
		ZoomPolygon		_currentPolygon;

		public ZoomPortView CurrentZoomLevel
		{
			get { return _currentZoomLevel; }
			set
			{
				if( value == _currentZoomLevel)
					return;

				_currentZoomLevel = value;
				UpdateCurrentPolygon();
				NotifyPropertyChanged(() => CurrentZoomLevel);
			}
		}

		public ZoomPolygon CurrentZoomPolygon
		{
			get { return _currentPolygon; }
			protected set
			{
				if (value == _currentPolygon)
					return;

				_currentPolygon = value;
				NotifyPropertyChanged(() => CurrentZoomPolygon);
			}
		}

		void UpdateCurrentPolygon()
		{
			if (_currentZoomLevel == null)
				CurrentZoomPolygon = null;
			else
			{
				CurrentZoomPolygon	= ZoomPolygon.Parse(_originalPolygonPoints, _currentZoomLevel);	//, out pointsOutPortView);
			}
		}

		public static ZoomPolygonOptimizeTest Instance
		{
			get
			{
				if (_instance == null)
					_instance = new ZoomPolygonOptimizeTest();
				return _instance;
			}
		}

		public static ZoomPolygonOptimizeTest GetInstance()
		{
			return Instance;
		}

		public ZoomPolygonOptimizeTest()
		{
			CreateOriginalPolygonPoints();
			_currentZoomLevel = _zoomPortViews[0];
			UpdateCurrentPolygon();
		}

		

		public double ZoomRatio
		{
			get {  return _zoomRatio; }
			set
			{
				if( value == _zoomRatio)
					return;

				_zoomRatio = value;
				ApplyRatio();
				UpdateCurrentPolygon();
				NotifyPropertyChanged(() => ZoomRatio);
			}
		}

		private void ApplyRatio()
		{
			foreach(ZoomPortView pv in _zoomPortViews)
				pv.ZoomRatio = _zoomRatio;
		}


		public List<ZoomPortView> ZoomLevels
		{
			get {  return _zoomPortViews; }
		}

		public int OriginalPolygonPointsCount
		{
			get {  return _originalPolygonPoints.Length; }
		}


		private void CreateOriginalPolygonPoints()
		{
			string[]	pointsXY	= _strPolygonPoints.Split(' ');
			string[]	coordinates;
			double		x,
						y;
			int			pointIndex	= 0;

			_originalPolygonPoints	= new Point[pointsXY.Length];

			for(int ndx = 0; ndx < pointsXY.Length; ndx++)
			{
				string		pointCoord	= pointsXY[ndx];

				coordinates	= pointCoord.Split(',');

				// Note: our string is formatted as Lon/Lat(!)... tranform this to Lat/Lon
				// ** 140914: keep Lon/Lat
				if( ! double.TryParse(coordinates[0], out x))
					continue;

				if (!double.TryParse(coordinates[1], out y))
					continue;

				_originalPolygonPoints[pointIndex++]	= new Point(x, y);
			}

		}

		Point[]  _originalPolygonPoints;
		
	}
}
