﻿using isosoft.root;
using System.Windows;

namespace PolygonOptimizer
{
	public class ZoomPortView : XObjectNotifier
	{
		Point	_topLeft		= new Point(),
				_bottomRight	= new Point();
		double	_zoomRatio		= 0.1;
		string	_name			= "";

		public ZoomPortView()
		{
		}

		public ZoomPortView(string name, Point topLeft, Point bottomRight, double zoomRatio)
		{
			_name			= name;
			_topLeft		= topLeft;
			_bottomRight	= bottomRight;
			_zoomRatio		= zoomRatio;
		}

		public string Name
		{
			get {  return _name; }
			set
			{
				if( value == _name)
					return;

				_name = value;
				NotifyPropertyChanged(() => Name);
			}
		}

		public double ZoomRatio
		{
			get { return _zoomRatio; }
			set
			{
				if (value < 0.0 || value == _zoomRatio)
					return;

				_zoomRatio = value;
				NotifyPropertyChanged(() => ZoomRatio);
			}
		}

		public Point TopLeft
		{
			get { return _topLeft; }
			set
			{
				if( value == _topLeft || value == null)
					return;
				_topLeft = value;
				NotifyPropertyChanged(() => TopLeft);
				RaiseDimensionChanged();
			}
		}

		public Point BottomRight
		{
			get { return _bottomRight; }
			set
			{
				if( value == _bottomRight || value == null)
					return;
				_bottomRight = value;
				NotifyPropertyChanged(() => BottomRight);
				RaiseDimensionChanged();
			}
		}

		protected void RaiseDimensionChanged()
		{
			NotifyPropertyChanged(() => Width);
			NotifyPropertyChanged(() => Height);
		}

		public double Width
		{
			get { return _topLeft.X - _bottomRight.X; }
		}

		public double Height
		{
			get { return _topLeft.Y - _bottomRight.Y; }
		}

		public bool IsPointInPortview(Point point)
		{
			if (point.X >= _topLeft.X && point.X <= _bottomRight.X && point.Y <= _topLeft.Y && point.Y >= _bottomRight.Y)
				return true;
			return false;
		}
	}
}
